﻿namespace BandLoadingBug
{
    using System;
    using System.Collections.Generic;
    using System.Data;
    using Infragistics.Win.UltraWinDataSource;
    using Infragistics.Win.UltraWinGrid;

    class MyUltraDataSource : UltraDataSource
    {
        private readonly DataSet dataSet;
        private readonly object lockObject = new object();
        private readonly IDictionary<string, IList<DataRow>> trpKeyToTrpDataRows = new Dictionary<string, IList<DataRow>>();
        private readonly IDictionary<string, int> bandToGridRow = new Dictionary<string, int>();
        private readonly IDictionary<int, string> gridRowToBand = new Dictionary<int, string>();

        public MyUltraDataSource(DataSet dataSet)
        {
            this.dataSet = dataSet;
            Init();
        }

        private void Init()
        {
            Band.Columns.Add("ProductType");
            CellDataRequested += OnCellDataRequested;

            foreach (DataTable table in dataSet.Tables)
            {
                if (!table.TableName.Contains(Form1.ChildNamePrefix)) // do not add child bands at this level
                {
                    UltraDataBand band = Band.ChildBands.Add(table.TableName);
                    foreach (DataColumn column in table.Columns)
                    {
                        band.Columns.Add(column.ColumnName, column.DataType);
                    }                    
                }
            }
        }

        private void OnCellDataRequested(object sender, CellDataRequestedEventArgs e)
        {
            int rowIndex = e.Row.Index;
            string bandKey = e.Row.Band.Key;

            if (bandKey == "ProductType")
            {
                lock (lockObject)
                {
                    if (dataSet.Tables.Contains(bandKey))
                    {
                        DataTable table = dataSet.Tables[bandKey];
                        if ( table.Rows.Count > rowIndex && table.Columns.Contains(e.Column.Key))
                        {
                            if (e.Row.Band.Key.Contains(Form1.ChildNamePrefix))
                            {
                                string tradeId = dataSet.Tables[e.Row.Band.ParentBand.Key].Rows[e.Row.ParentRow.Index]["ID"].ToString();
                                string trpKey = CreateTrpKey(e.Row.Band.ParentBand.Key, tradeId);
                                if (trpKeyToTrpDataRows.ContainsKey(trpKey))
                                {
                                    IList<DataRow> dataRows = trpKeyToTrpDataRows[trpKey];
                                    if (dataRows.Count > rowIndex)
                                    {
                                        e.Data = dataRows[rowIndex][e.Column.Key];
                                    }
                                }
                            }
                            else
                            {
                                e.Data = table.Rows[rowIndex][e.Column.Key];
                            }
                        }
                    }
                }
            }
            e.CacheData = false;
        }

        public void BeforeRowExpanded(object sender, CancelableRowEventArgs e)
        {
            string bandKey = e.Row.Band.Key;

            if (Band.ChildBands.Contains(bandKey))
            {

            }
        }

        public void UpdateData(DataSet updateSet)
        {
            lock(lockObject)
            {
                foreach (DataTable updateTable in updateSet.Tables)
                {
                    DataTable targetTable = dataSet.Tables[updateTable.TableName];
                    bool isChildTable = false;
                    DataTable parentTable = null;

                    if ( updateTable.TableName.Contains(Form1.ChildNamePrefix))
                    {
                        isChildTable = true;
                        string parentTableName = updateTable.TableName.Replace(Form1.ChildNamePrefix, string.Empty);
                        parentTable = dataSet.Tables[parentTableName];
                    }

                    IDictionary<DataRow, DataRow> trpRowToParentRowAdds = new Dictionary<DataRow, DataRow>();
                    foreach(DataRow updateRow in updateTable.Rows)
                    {
                        DataRow addedRow = AddRow(targetTable, updateRow);
                        DataRow parentRow = null;
                        if ( isChildTable)
                        {
                            parentRow = parentTable.Rows.Find(updateRow["ID"]);
                            if ( parentRow == null)
                            {
                                continue;
                            }
                        
                            trpRowToParentRowAdds[addedRow] = parentRow;
                        }
                    }


                    targetTable.AcceptChanges();
                    if (!isChildTable)
                    {
                        if (!bandToGridRow.ContainsKey(updateTable.TableName))
                        {
                            AddNewBandAndGridmappings(updateTable.TableName);
                            Rows.Add(false, new object[] {updateTable.TableName});
                        }

                        int gridRowIndex = bandToGridRow[updateTable.TableName];
                        Rows[gridRowIndex].GetChildRows(updateTable.TableName).SetCount(updateTable.Rows.Count);
                    }
                    else
                    {
                        foreach (KeyValuePair<DataRow, DataRow> rowPair in trpRowToParentRowAdds)
                        {
                            string trpKey = CreateTrpKey(parentTable.TableName, rowPair.Key["ID"].ToString());
                            int trpParentGridRowIndex = bandToGridRow[parentTable.TableName];
                            UltraDataRowsCollection parentBandTradeCollection = Rows[trpParentGridRowIndex].GetChildRows(parentTable.TableName);
                            
                            int trpParentRowIndex = parentTable.Rows.IndexOf(rowPair.Value);
                            UltraDataRow trpParentUltraDataRow = parentBandTradeCollection[trpParentRowIndex];
                            if (!trpParentUltraDataRow.Band.ChildBands.Exists(updateTable.TableName))
                            {
                                AddTrpChildBand(parentBandTradeCollection[trpParentRowIndex], updateTable.TableName);
                            }

                            UltraDataRowsCollection trpBandTradeCollection = parentBandTradeCollection[trpParentRowIndex].GetChildRows(updateTable.TableName);
                            trpBandTradeCollection.SetCount(trpBandTradeCollection.Count + 1);

                            if (trpKeyToTrpDataRows.ContainsKey(trpKey))
                            {
                                trpKeyToTrpDataRows[trpKey].Add(rowPair.Key);
                            }
                            else
                            {
                                trpKeyToTrpDataRows[trpKey] = new List<DataRow> { rowPair.Key };
                            }
                        }
                    }
                }
            }
        }

        private DataRow AddRow(DataTable targetTable, DataRow sourceRow)
        {
            DataRow row = targetTable.NewRow();
            foreach (DataColumn column in targetTable.Columns)
            {
                if (sourceRow.Table.Columns.Contains(column.ColumnName))
                {
                    row[column] = sourceRow[column.ColumnName];
                }
            }
            targetTable.Rows.Add(row);
            return row;
        }

        private static string CreateTrpKey(string parentProduct, string trpTradeId)
        {
            return String.Format("{0}___{1}", parentProduct, trpTradeId);
        }

        private void AddNewBandAndGridmappings(string product)
        {
            if (bandToGridRow.ContainsKey(product))
            {
                throw new ArgumentException("product already exists in the mappings");
            }

            int gridRowIndex = bandToGridRow.Count;
            bandToGridRow[product] = gridRowIndex;
            gridRowToBand[gridRowIndex] = product;
        }

        private void AddTrpChildBand(UltraDataRow targetRow, string trpProduct)
        {
            DataTable dataTable = dataSet.Tables[trpProduct];

            UltraDataBand band = targetRow.Band.ChildBands.Add(dataTable.TableName);
            foreach (DataColumn column in dataTable.Columns)
            {
                band.Columns.Add(column.ColumnName, column.DataType);
            }
        }
    }
}
